@@ -18,18 +18,18 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 """
 
-import sys, os
-from optparse import OptionParser, make_option
-
-from stgit.commands.common import *
-from stgit.utils import *
-from stgit.out import *
-from stgit import stack, git, templates
-
-
-help = 'exports patches to a directory'
-usage = """%prog [options] [<patch1>] [<patch2>] [<patch3>..<patch4>]
+import os
+import sys
+from stgit.argparse import opt
+from stgit.commands import common
+from stgit import argparse, git, templates
+from stgit.out import out
+from stgit.lib import git as gitlib
 
+help = 'Export patches to a directory'
+kind = 'patch'
+usage = ['[options] [<patch1>] [<patch2>] [<patch3>..<patch4>]']
+description = """
 Export a range of applied patches to a given directory (defaults to
 'patches-<branch>') in a standard unified GNU diff format. A template
 file (defaulting to '.git/patchexport.tmpl' or
@@ -46,38 +46,39 @@ file:
   %(authemail)s   - author's e-mail
   %(authdate)s    - patch creation date
   %(commname)s    - committer's name
-  %(commemail)s   - committer's e-mail
-"""
+  %(commemail)s   - committer's e-mail"""
 
-directory = DirectoryHasRepository()
-options = [make_option('-d', '--dir',
-                       help = 'export patches to DIR instead of the default'),
-           make_option('-p', '--patch',
-                       help = 'append .patch to the patch names',
-                       action = 'store_true'),
-           make_option('-e', '--extension',
-                       help = 'append .EXTENSION to the patch names'),
-           make_option('-n', '--numbered',
-                       help = 'prefix the patch names with order numbers',
-                       action = 'store_true'),
-           make_option('-t', '--template', metavar = 'FILE',
-                       help = 'Use FILE as a template'),
-           make_option('-b', '--branch',
-                       help = 'use BRANCH instead of the default one'),
-           make_option('-O', '--diff-opts',
-                       help = 'options to pass to git-diff'),
-           make_option('-s', '--stdout',
-                       help = 'dump the patches to the standard output',
-                       action = 'store_true')]
+args = [argparse.patch_range(argparse.applied_patches,
+                             argparse.unapplied_patches,
+                             argparse.hidden_patches)]
+options = [
+    opt('-d', '--dir', args = [argparse.dir],
+        short = 'Export patches to DIR instead of the default'),
+    opt('-p', '--patch', action = 'store_true',
+        short = 'Append .patch to the patch names'),
+    opt('-e', '--extension',
+        short = 'Append .EXTENSION to the patch names'),
+    opt('-n', '--numbered', action = 'store_true',
+        short = 'Prefix the patch names with order numbers'),
+    opt('-t', '--template', metavar = 'FILE', args = [argparse.files],
+        short = 'Use FILE as a template'),
+    opt('-b', '--branch', args = [argparse.stg_branches],
+        short = 'Use BRANCH instead of the default branch'),
+    opt('-s', '--stdout', action = 'store_true',
+        short = 'Dump the patches to the standard output'),
+    ] + argparse.diff_opts_option()
 
+directory = common.DirectoryHasRepositoryLib()
 
 def func(parser, options, args):
     """Export a range of patches.
     """
+    stack = directory.repository.get_stack(options.branch)
+
     if options.dir:
         dirname = options.dir
     else:
-        dirname = 'patches-%s' % crt_series.get_name()
+        dirname = 'patches-%s' % stack.name
         directory.cd_to_topdir()
 
     if not options.branch and git.local_changes():
@@ -89,21 +90,16 @@ def func(parser, options, args):
             os.makedirs(dirname)
         series = file(os.path.join(dirname, 'series'), 'w+')
 
-    if options.diff_opts:
-        diff_flags = options.diff_opts.split()
-    else:
-        diff_flags = []
-
-    applied = crt_series.get_applied()
-    unapplied = crt_series.get_unapplied()
+    applied = stack.patchorder.applied
+    unapplied = stack.patchorder.unapplied
     if len(args) != 0:
-        patches = parse_patches(args, applied + unapplied, len(applied))
+        patches = common.parse_patches(args, applied + unapplied, len(applied))
     else:
         patches = applied
 
     num = len(patches)
     if num == 0:
-        raise CmdException, 'No patches applied'
+        raise common.CmdException, 'No patches applied'
 
     zpadding = len(str(num))
     if zpadding < 2:
@@ -119,7 +115,7 @@ def func(parser, options, args):
 
     # note the base commit for this series
     if not options.stdout:
-        base_commit = crt_series.get_patch(patches[0]).get_bottom()
+        base_commit = stack.patches.get(patches[0]).commit.sha1
         print >> series, '# This series applies on GIT commit %s' % base_commit
 
     patch_no = 1;
@@ -136,25 +132,27 @@ def func(parser, options, args):
             print >> series, pname
 
         # get the patch description
-        patch = crt_series.get_patch(p)
+        patch = stack.patches.get(p)
+        cd = patch.commit.data
 
-        descr = patch.get_description().strip()
+        descr = cd.message.strip()
         descr_lines = descr.split('\n')
 
         short_descr = descr_lines[0].rstrip()
         long_descr = reduce(lambda x, y: x + '\n' + y,
                             descr_lines[1:], '').strip()
 
-        tmpl_dict = {'description': patch.get_description().rstrip(),
+        diff = stack.repository.diff_tree(cd.parent.data.tree, cd.tree, options.diff_flags)
+
+        tmpl_dict = {'description': descr,
                      'shortdescr': short_descr,
                      'longdescr': long_descr,
-                     'diffstat': git.diffstat(rev1 = patch.get_bottom(),
-                                              rev2 = patch.get_top()),
-                     'authname': patch.get_authname(),
-                     'authemail': patch.get_authemail(),
-                     'authdate': patch.get_authdate(),
-                     'commname': patch.get_commname(),
-                     'commemail': patch.get_commemail()}
+                     'diffstat': gitlib.diffstat(diff).rstrip(),
+                     'authname': cd.author.name,
+                     'authemail': cd.author.email,
+                     'authdate': cd.author.date.isoformat(),
+                     'commname': cd.committer.name,
+                     'commemail': cd.committer.email}
         for key in tmpl_dict:
             if not tmpl_dict[key]:
                 tmpl_dict[key] = ''
@@ -162,10 +160,10 @@ def func(parser, options, args):
         try:
             descr = tmpl % tmpl_dict
         except KeyError, err:
-            raise CmdException, 'Unknown patch template variable: %s' \
+            raise common.CmdException, 'Unknown patch template variable: %s' \
                   % err
         except TypeError:
-            raise CmdException, 'Only "%(name)s" variables are ' \
+            raise common.CmdException, 'Only "%(name)s" variables are ' \
                   'supported in the patch template'
 
         if options.stdout:
@@ -175,13 +173,11 @@ def func(parser, options, args):
 
         if options.stdout and num > 1:
             print '-'*79
-            print patch.get_name()
+            print patch.name
             print '-'*79
 
         f.write(descr)
-        f.write(git.diff(rev1 = patch.get_bottom(),
-                         rev2 = patch.get_top(),
-                         diff_flags = diff_flags))
+        f.write(diff)
         if not options.stdout:
             f.close()
         patch_no += 1
