@@ -17,57 +17,55 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 """
 
 import sys, os
-from optparse import OptionParser, make_option
-
+from stgit.argparse import opt
 from stgit.commands.common import *
 from stgit.utils import *
 from stgit.out import *
-from stgit import stack, git
-
-
-help = 'show the files modified by a patch (or the current patch)'
-usage = """%prog [options] [<patch>]
+from stgit import argparse, stack, git
+from stgit.lib import git as gitlib
 
+help = 'Show the files modified by a patch (or the current patch)'
+kind = 'patch'
+usage = ['[options] [[<branch>:]<patch>]']
+description = """
 List the files modified by the given patch (defaulting to the current
 one). Passing the '--stat' option shows the diff statistics for the
 given patch. Note that this command doesn't show the files modified in
 the working tree and not yet included in the patch by a 'refresh'
 command. Use the 'diff' or 'status' commands for these files."""
 
-directory = DirectoryHasRepository()
-options = [make_option('-s', '--stat',
-                       help = 'show the diff stat',
-                       action = 'store_true'),
-           make_option('-b', '--branch',
-                       help = 'use BRANCH instead of the default one'),
-           make_option('-O', '--diff-opts',
-                       help = 'options to pass to git-diff'),
-           make_option('--bare',
-                       help = 'bare file names (useful for scripting)',
-                       action = 'store_true')]
+args = [argparse.applied_patches, argparse.unapplied_patches,
+        argparse.hidden_patches]
+options = [
+    opt('-s', '--stat', action = 'store_true',
+        short = 'Show the diffstat'),
+    opt('--bare', action = 'store_true',
+        short = 'Bare file names (useful for scripting)'),
+    ] + argparse.diff_opts_option()
 
+directory = DirectoryHasRepository(log = False)
 
 def func(parser, options, args):
     """Show the files modified by a patch (or the current patch)
     """
     if len(args) == 0:
-        patch = ''
+        patch = 'HEAD'
     elif len(args) == 1:
         patch = args[0]
     else:
         parser.error('incorrect number of arguments')
 
-    rev1 = git_id(crt_series, '%s//bottom' % patch)
-    rev2 = git_id(crt_series, '%s//top' % patch)
+    rev1 = git_id(crt_series, '%s^' % patch)
+    rev2 = git_id(crt_series, '%s' % patch)
 
     if options.stat:
-        out.stdout_raw(git.diffstat(rev1 = rev1, rev2 = rev2) + '\n')
+        output = gitlib.diffstat(git.diff(rev1 = rev1, rev2 = rev2,
+                                          diff_flags = options.diff_flags))
     elif options.bare:
-        out.stdout_raw(git.barefiles(rev1, rev2) + '\n')
+        output = git.barefiles(rev1, rev2)
     else:
-        if options.diff_opts:
-            diff_flags = options.diff_opts.split()
-        else:
-            diff_flags = []
-
-        out.stdout_raw(git.files(rev1, rev2, diff_flags = diff_flags) + '\n')
+        output = git.files(rev1, rev2, diff_flags = options.diff_flags)
+    if output:
+        if not output.endswith('\n'):
+            output += '\n'
+        out.stdout_raw(output)
