@@ -16,29 +16,30 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 """
 
 import sys, os
-from optparse import OptionParser, make_option
 from pydoc import pager
-
+from stgit.argparse import opt
 from stgit.commands.common import *
 from stgit.utils import *
 from stgit.out import *
-from stgit import stack, git
-
-
-help = 'show the applied patches modifying a file'
-usage = """%prog [options] [<files or dirs>]
+from stgit import argparse, stack, git
 
+help = 'Show the applied patches modifying a file'
+kind = 'stack'
+usage = ['[options] [<files or dirs>]']
+description = """
 Show the applied patches modifying the given files. Without arguments,
 it shows the patches affected by the local tree modifications. The
 '--diff' option also lists the patch log and the diff for the given
 files."""
 
-directory = DirectoryHasRepository()
-options = [make_option('-d', '--diff',
-                       help = 'show the diff for the given files',
-                       action = 'store_true'),
-           make_option('-b', '--branch',
-                       help = 'use BRANCH instead of the default one')]
+args = [argparse.known_files]
+options = [
+    opt('-d', '--diff', action = 'store_true',
+        short = 'Show the diff for the given files'),
+    opt('-b', '--branch', args = [argparse.stg_branches],
+        short = 'Use BRANCH instead of the default branch')]
+
+directory = DirectoryHasRepository(log = False)
 
 diff_tmpl = \
           '-------------------------------------------------------------------------------\n' \
