@@ -16,38 +16,38 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 """
 
 import sys, os
-from optparse import OptionParser, make_option
-
+from stgit.argparse import opt
 from stgit.commands.common import *
 from stgit.utils import *
-from stgit import stack, git
-
-
-help = 'move the stack base to another point in history'
-usage = """%prog [options] <new-base-id>
+from stgit import argparse, stack, git
 
+help = 'Move the stack base to another point in history'
+kind = 'stack'
+usage = ['[options] <new-base-id>']
+description = """
 Pop all patches from current stack, move the stack base to the given
 <new-base-id> and push the patches back.
 
 If you experience merge conflicts, resolve the problem and continue
 the rebase by executing the following sequence:
 
-        $ stg resolved -a [-i]
+        $ git add --update
         $ stg refresh
         $ stg goto top-patch
 
 Or if you want to skip that patch:
 
-        $ stg push --undo
+        $ stg undo --hard
         $ stg push next-patch..top-patch"""
 
-directory = DirectoryGotoToplevel()
-options = [make_option('-n', '--nopush',
-                       help = 'do not push the patches back after rebasing',
-                       action = 'store_true'),
-           make_option('-m', '--merged',
-                       help = 'check for patches merged upstream',
-                       action = 'store_true')]
+args = [argparse.commit]
+options = [
+    opt('-n', '--nopush', action = 'store_true',
+        short = 'Do not push the patches back after rebasing'),
+    opt('-m', '--merged', action = 'store_true',
+        short = 'Check for patches merged upstream')]
+
+directory = DirectoryGotoToplevel(log = True)
 
 def func(parser, options, args):
     """Rebase the current stack
