@@ -29,9 +29,9 @@ test_expect_success \
 test_expect_success \
     'Port those patches to orig tree' \
     '(cd foo &&
-      GIT_DIR=../bar/.git git-format-patch --stdout \
-          $(cd ../bar && stg id base@master)..HEAD |
-      git-am -3 -k
+      GIT_DIR=../bar/.git git format-patch --stdout \
+          $(cd ../bar && stg id master:{base})..HEAD |
+      git am -3 -k
      )
     '
 
@@ -49,13 +49,13 @@ test_expect_success \
 
 test_expect_success \
     'Pull those patches applied upstream' \
-    "(cd bar && stg push --undo && stg push --all --merged
+    "(cd bar && stg undo && stg push --all --merged
      )
 "
 
 test_expect_success \
     'Check that all went well' \
-    "diff -u foo/file bar/file
+    "test_cmp foo/file bar/file
 "
 
 test_done
