@@ -3,10 +3,10 @@
 # Copyright (c) 2006 Catalin Marinas
 #
 
-test_description='Exercise push --undo with missing files.
+test_description='Exercise stg undo with push of missing files.
 
 Test the case where a patch fails to be pushed because it modifies a
-missing file. The "push --undo" command has to be able to revert it.
+missing file. The "stg undo" command has to be able to revert it.
 '
 
 . ./test-lib.sh
@@ -21,7 +21,7 @@ test_expect_success \
 	'
 	stg new foo -m foo &&
 	echo foo > test &&
-	stg add test &&
+	git add test &&
 	stg refresh
 	'
 
@@ -30,7 +30,7 @@ test_expect_success \
 	'
 	stg new bar -m bar &&
 	echo bar > test &&
-	stg add test &&
+	git add test &&
 	stg refresh
 	'
 
@@ -43,28 +43,28 @@ test_expect_success \
 test_expect_success \
 	'Push the second patch with conflict' \
 	'
-	! stg push bar
+	conflict stg push bar
 	'
 
 test_expect_success \
 	'Undo the previous push' \
 	'
-	stg push --undo
+	stg undo --hard
 	'
 
 test_expect_success \
 	'Check the push after undo fails as well' \
 	'
-	! stg push bar
+	conflict stg push bar
 	'
 
 test_expect_success \
 	'Undo with disappeared newborn' \
 	'
 	touch newfile &&
-	stg add newfile &&
+	git add newfile &&
 	rm newfile &&
-	stg push --undo
+	stg undo --hard
 	'
 
 test_done
