@@ -27,20 +27,20 @@ test_expect_success \
 	'Rebase to previous commit' \
 	'
 	stg rebase master~1 &&
-	test `stg id base@stack` = `git rev-parse master~1` &&
-	test `stg applied | wc -l` = 1
+	test `stg id stack:{base}` = `git rev-parse master~1` &&
+	test `stg series --applied -c` = 1
 	'
 
 test_expect_success \
 	'Attempt rebase to non-existing commit' \
 	'
-	! stg rebase not-a-ref
+	command_error stg rebase not-a-ref
 	'
 
 test_expect_success \
 	'Check patches were re-applied' \
 	'
-	test $(stg applied | wc -l) = 1
+	test $(stg series --applied -c) = 1
 	'
 
 test_done
